@@ -1,4 +1,6 @@
 package POE::Component::Server::NSCA;
+$POE::Component::Server::NSCA::VERSION = '0.10';
+#ABSTRACT: a POE Component that implements NSCA daemon functionality
 
 use strict;
 use warnings;
@@ -7,15 +9,13 @@ use Carp;
 use Net::Netmask;
 use Math::Random;
 use POE qw(Wheel::SocketFactory Wheel::ReadWrite Filter::Stream);
-use vars qw($VERSION);
-
-$VERSION='0.08';
 
 use constant MAX_INPUT_BUFFER =>        2048    ; # /* max size of most buffers we use */
 use constant MAX_HOST_ADDRESS_LENGTH => 256     ; # /* max size of a host address */
 use constant MAX_HOSTNAME_LENGTH =>     64      ;
 use constant MAX_DESCRIPTION_LENGTH =>  128;
-use constant MAX_PLUGINOUTPUT_LENGTH => 512;
+use constant OLD_PLUGINOUTPUT_LENGTH => 512;
+use constant MAX_PLUGINOUTPUT_LENGTH => 4096;
 use constant MAX_PASSWORD_LENGTH =>     512;
 use constant TRANSMITTED_IV_SIZE =>     128;
 use constant SIZEOF_U_INT32_T   => 4;
@@ -25,6 +25,7 @@ use constant SIZEOF_INIT_PACKET => TRANSMITTED_IV_SIZE + SIZEOF_U_INT32_T;
 use constant PROBABLY_ALIGNMENT_ISSUE => 4;
 
 use constant SIZEOF_DATA_PACKET => SIZEOF_INT16_T + SIZEOF_U_INT32_T + SIZEOF_U_INT32_T + SIZEOF_INT16_T + MAX_HOSTNAME_LENGTH + MAX_DESCRIPTION_LENGTH + MAX_PLUGINOUTPUT_LENGTH + PROBABLY_ALIGNMENT_ISSUE;
+use constant SIZEOF_OLD_PACKET  => SIZEOF_INT16_T + SIZEOF_U_INT32_T + SIZEOF_U_INT32_T + SIZEOF_INT16_T + MAX_HOSTNAME_LENGTH + MAX_DESCRIPTION_LENGTH + OLD_PLUGINOUTPUT_LENGTH + PROBABLY_ALIGNMENT_ISSUE;
 
 use constant ENCRYPT_NONE =>            0       ; # /* no encryption */
 use constant ENCRYPT_XOR =>             1       ; # /* not really encrypted, just obfuscated */
@@ -63,31 +64,31 @@ eval {
 # Lookups for loading.
 my %mcrypts =   (       ENCRYPT_NONE,		"none",
 			ENCRYPT_XOR,		"xor",
-			ENCRYPT_DES,            "des",
-                        ENCRYPT_3DES,           "3des",
-                        ENCRYPT_CAST128,        "cast-128",
-                        ENCRYPT_CAST256,        "cast-256",
-                        ENCRYPT_XTEA,           "xtea",
-                        ENCRYPT_3WAY,           "threeway",
-                        ENCRYPT_BLOWFISH,       "blowfish",
-                        ENCRYPT_TWOFISH,        "twofish",
-                        ENCRYPT_LOKI97,         "loki97",
-                        ENCRYPT_RC2,            "rc2",
-                        ENCRYPT_ARCFOUR,        "arcfour",
-                        ENCRYPT_RC6,            "rc6",
-                        ENCRYPT_RIJNDAEL128,    "rijndael-128",
-                        ENCRYPT_RIJNDAEL192,    "rijndael-192",
-                        ENCRYPT_RIJNDAEL256,    "rijndael-256",
-                        ENCRYPT_MARS,           "mars",
-                        ENCRYPT_PANAMA,         "panama",
-                        ENCRYPT_WAKE,           "wake",
-                        ENCRYPT_SERPENT,        "serpent",
-                        ENCRYPT_IDEA,           "idea",
-                        ENCRYPT_ENIGMA,         "engima",
-                        ENCRYPT_GOST,           "gost",
-                        ENCRYPT_SAFER64,        "safer-sk64",
-                        ENCRYPT_SAFER128,       "safer-sk128",
-                        ENCRYPT_SAFERPLUS,      "saferplus",
+			ENCRYPT_DES,            "DES",
+                        ENCRYPT_3DES,           "3DES",
+                        ENCRYPT_CAST128,        "CAST_128",
+                        ENCRYPT_CAST256,        "CAST-256",
+                        ENCRYPT_XTEA,           "XTEA",
+                        ENCRYPT_3WAY,           "THREEWAY",
+                        ENCRYPT_BLOWFISH,       "BLOWFISH",
+                        ENCRYPT_TWOFISH,        "TWOFISH",
+                        ENCRYPT_LOKI97,         "LOKI97",
+                        ENCRYPT_RC2,            "RC2",
+                        ENCRYPT_ARCFOUR,        "ARCFOUR",
+                        ENCRYPT_RC6,            "RC6",
+                        ENCRYPT_RIJNDAEL128,    "RIJNDAEL_128",
+                        ENCRYPT_RIJNDAEL192,    "RIJNDAEL_192",
+                        ENCRYPT_RIJNDAEL256,    "RIJNDAEL_256",
+                        ENCRYPT_MARS,           "MARS",
+                        ENCRYPT_PANAMA,         "PANAMA",
+                        ENCRYPT_WAKE,           "WAKE",
+                        ENCRYPT_SERPENT,        "SERPENT",
+                        ENCRYPT_IDEA,           "IDEA",
+                        ENCRYPT_ENIGMA,         "ENGIMA",
+                        ENCRYPT_GOST,           "GOST",
+                        ENCRYPT_SAFER64,        "SAFER_SK64",
+                        ENCRYPT_SAFER128,       "SAFER_SK128",
+                        ENCRYPT_SAFERPLUS,      "SAFERPLUS",
                 );
 
 sub spawn {
@@ -148,10 +149,11 @@ sub _start {
   $self->{session_id} = $_[SESSION]->ID();
   if ( $self->{alias} ) {
 	$kernel->alias_set( $self->{alias} );
-  } 
+  }
   else {
 	$kernel->refcount_increment( $self->{session_id} => __PACKAGE__ );
   }
+  #$self->{filter} = POE::Filter::Block->new( BlockSize => SIZEOF_DATA_PACKET );
   $self->{filter} = POE::Filter::Stream->new();
   $self->{listener} = POE::Wheel::SocketFactory->new(
       ( defined $self->{address} ? ( BindAddress => $self->{address} ) : () ),
@@ -249,14 +251,14 @@ sub _accept_client {
   );
 
   return unless $wheel;
-  
+
   my $id = $wheel->ID();
   my $time = time();
   my $iv = join '', random_uniform_integer(128,0,9);
   my $init_packet = $iv . pack 'N', $time;
-  $self->{clients}->{ $id } = 
-  { 
-	wheel    => $wheel, 
+  $self->{clients}->{ $id } =
+  {
+	wheel    => $wheel,
 	peeraddr => $peeraddr,
 	peerport => $peerport,
 	sockaddr => $sockaddr,
@@ -272,7 +274,7 @@ sub _accept_client {
 sub _conn_exists {
   my ($self,$wheel_id) = @_;
   return 0 unless $wheel_id and defined $self->{clients}->{ $wheel_id };
-  return 1; 
+  return 1;
 }
 
 sub _conn_error {
@@ -292,14 +294,17 @@ sub _conn_alarm {
 sub _conn_input {
   my ($kernel,$self,$packet,$id) = @_[KERNEL,OBJECT,ARG0,ARG1];
   return unless $self->_conn_exists( $id );
-  my $client = delete $self->{clients}->{ $id };
-  delete $client->{wheel};
+  my $client = $self->{clients}->{ $id };
   $kernel->alarm_remove( delete $client->{alarm} );
-  return unless length( $packet ) == SIZEOF_DATA_PACKET; # wrong packet size received
-  my $input = _decrypt( $packet, $self->{encryption}, $client->{iv}, $self->{password} );
+  my $data_packet_length = SIZEOF_DATA_PACKET;
+  if ( length( $packet ) == SIZEOF_OLD_PACKET ) {
+    $data_packet_length = SIZEOF_OLD_PACKET;
+  }
+  my $input = _decrypt( $packet, $self->{encryption}, $client->{iv}, $self->{password}, $data_packet_length );
   return unless $input; # something wrong with the decryption
   my $version = unpack 'n', substr $input, 0, 4;
-  return unless $version == 3; # Wrong version received
+  return unless $version == 3 or $client->{version_already_checked}; # Wrong version received
+  $client->{version_already_checked} = 1;
   my $crc32 = unpack 'N', substr $input, 4, 4;
   my $ts = unpack 'N', substr $input, 8, 4;
   my $rc = unpack 'n', substr $input, 12, 2;
@@ -308,13 +313,16 @@ sub _conn_input {
   my $checksum = _calculate_crc32( $firstbit . pack('N', 0) . $secondbit );
   my @data = unpack 'a[64]a[128]a[512]', substr $input, 14;
   s/\000.*$// for @data;
-  $client->{version} = $version;
-  $client->{crc32} = $crc32;
-  $client->{checksum} = $checksum;
-  $client->{return_code} = $rc;
-  $client->{$_} = shift @data for qw(host_name svc_description plugin_output);
-  delete $client->{$_} for qw(peerport sockaddr sockport ts iv);
-  $kernel->post( $_, $self->{sessions}->{$_}->{event}, $client, $self->{sessions}->{$_}->{context} )
+  my $result = {
+   version      => $version,
+   crc32        => $crc32,
+   checksum     => $checksum,
+   return_code  => $rc,
+   timestamp    => $ts,
+  };
+  $result->{$_} = shift @data for qw(host_name svc_description plugin_output);
+  $result->{$_} = $client->{$_} for qw(peeraddr peerport sockaddr sockport ts iv);
+  $kernel->post( $_, $self->{sessions}->{$_}->{event}, $result, $self->{sessions}->{$_}->{context} )
 	for keys %{ $self->{sessions} };
   return;
 }
@@ -344,7 +352,7 @@ sub _generate_crc32_table {
      for (my $j = 8; $j > 0; $j--) {
         if ($crc & 1) {
           $crc = ($crc >> 1) ^ $poly;
-        } 
+        }
 	else {
           $crc = ($crc >> 1);
         }
@@ -356,15 +364,15 @@ sub _generate_crc32_table {
 
 # central switchboard for encryption methods.
 sub _decrypt {
-  my ($data_packet_string, $encryption_method, $iv_salt, $password) = @_;
+  my ($data_packet_string, $encryption_method, $iv_salt, $password, $data_packet_length) = @_;
 
   my $crypted;
   if ($encryption_method == ENCRYPT_NONE) {
        $crypted = $data_packet_string;
-  } 
+  }
   elsif ($encryption_method == ENCRYPT_XOR) {
-       $crypted = _decrypt_xor($data_packet_string, $iv_salt, $password);
-  } 
+       $crypted = _decrypt_xor($data_packet_string, $iv_salt, $password, $data_packet_length);
+  }
   else {
        $crypted = _decrypt_mcrypt( $data_packet_string, $encryption_method, $iv_salt, $password );
   }
@@ -372,7 +380,7 @@ sub _decrypt {
 }
 
 sub _decrypt_xor {
-  my ($data_packet_string, $iv_salt, $password) = @_;
+  my ($data_packet_string, $iv_salt, $password, $data_packet_length) = @_;
 
   my @out = split(//, $data_packet_string);
   my @salt_iv = split(//, $iv_salt);
@@ -382,7 +390,7 @@ sub _decrypt_xor {
   my $x = 0;
 
   #/* rotate over IV we received from the server... */
-  while ($y < SIZEOF_DATA_PACKET) {
+  while ($y < $data_packet_length) {
      #/* keep rotating over IV */
      $out[$y] = $out[$y] ^ $salt_iv[$x % scalar(@salt_iv)];
 
@@ -393,7 +401,7 @@ sub _decrypt_xor {
   #/* rotate over password... */
   $y=0;
   $x=0;
-  while ($y < SIZEOF_DATA_PACKET){
+  while ($y < $data_packet_length){
      #/* keep rotating over password */
      $out[$y] = $out[$y] ^ $salt_pw[$x % scalar(@salt_pw)];
 
@@ -414,7 +422,7 @@ sub _decrypt_mcrypt {
         my $routine = $mcrypts{$encryption_method};
         eval {
            # This sometimes dies with 'mcrypt is not of type MCRYPT'.
-           my $td = Mcrypt->new( algorithm => $routine, mode => 'cfb', verbose => 0 );
+           my $td = Mcrypt->new( algorithm => &{\&{"Mcrypt::".$routine}}(), mode => Mcrypt::CFB(), verbose => 0 );
            my $key = $password;
            my $iv = substr $iv_salt, 0, $td->{IV_SIZE};
            if( defined( $td ) ){
@@ -428,6 +436,8 @@ sub _decrypt_mcrypt {
         };
 	warn "$@\n" if $@;
      }
+  } else {
+    warn "Mcrypt module missing\n";
   }
   return $crypted;
 }
@@ -436,10 +446,18 @@ sub _decrypt_mcrypt {
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
 POE::Component::Server::NSCA - a POE Component that implements NSCA daemon functionality
 
+=head1 VERSION
+
+version 0.10
+
 =head1 SYNOPSIS
 
   use strict;
@@ -616,8 +634,10 @@ ARG0 will contain a hashref with the following key/values:
  'return_code', the result code of the check;
  'plugin_output', any output from the check plugin;
  'peeraddr', the IP address of the client that gave us the check information;
+ 'peerport', the IP address of the client that gave us the check information;
  'crc32', the checksum provided by the client;
  'checksum', the checksum as the poco calculated it;
+ 'timestamp', the clients timestamp;
 
 =item C<ARG1>
 
@@ -626,22 +646,12 @@ registered.
 
 =back
 
-=head1 AUTHOR
-
-Chris C<BinGOs> Williams <chris@bingosnet.co.uk>
+=head1 PROVENANCE
 
 Based on L<Net::Nsca> by P Kent
 
 Which was originally derived from work by Ethan Galstad.
 
-See the LICENSE file for details.
-
-=head1 LICENSE
-
-Copyright E<copy> Chris Williams, P Kent and Ethan Galstad.
-
-This module may be used, modified, and distributed under the same terms as Perl itself. Please see the license that came with your Perl distribution for details.
-
 =head1 SEE ALSO
 
 L<POE>
@@ -654,4 +664,15 @@ L<Mcrypt>
 
 L<http://www.nagios.org/>
 
+=head1 AUTHOR
+
+Chris Williams <chris@bingosnet.co.uk>
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2017 by Chris Williams, P Kent and Ethan Galstad.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
 =cut
